"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class Host:
    """This class represents a host object

    :param name: Name of the host
    :type name: str
    :param alias: Alias of the host
    :type alias: str
    :param address: Address of the host
    :type address: str
    :param activate: Is the host activated?
    :type activate: bool
    :param id_unique: Id of the host
    :type id_unique: int
    """

    def __init__(self, name, alias, address, activate, id_unique):
        self.name = name
        self.alias = alias
        self.address = address
        self.activate = activate
        self.id_unique = id_unique


class HostParam(enum.Enum):
    """This class represents a parameter of a host"""
    COORDS_2D = "2d_coords"
    """2d coordinates of the host. Format: (0.0, 0.0) (str)"""
    COORDS_3D = "3d_coords"
    """3d coordinates of the host. Format: (0.0, 0.0, 0.0) (str)"""
    ACTION_URL = "action_url"
    """Action URL. (str)"""
    ACTIVATE = "activate"
    """Is the Host activated? (bool)"""
    ACTIVE_CHECKS_ENABLED = "active_checks_enabled"
    """Are active checks enabled? (:ref:`class_general_three_way_option`)"""
    ADDRESS = "address"
    """Address of the host. Can be domain or IP. (str)"""
    ALIAS = "alias"
    """Alias of the host. (str)"""
    CHECK_COMMAND = "check_command"
    """Check command the host shoud use. (str)"""
    CHECK_COMMAND_ARGUMENTS = "check_command_arguments"
    """Check command arguments. (list of str)"""
    CHECK_INTERVAL = "check_interval"
    """Interval between the check execution. (int)"""
    CHECK_FRESHNESS = "check_freshness"
    """Determines whether or not freshness checks are enabled. When enabled the monitoring engine will trigger an 
    active check when the last passive result is older than the value defined in the threshold.
    (:ref:`class_general_three_way_option`)"""
    CHECK_PERIOD = "check_period"
    """Check period in which the host should be checked (str)"""
    CONTACT_ADDITIVE_INHERITANCE = "contact_additive_inheritance"
    """Enables or disables contact additive inheritance. (bool)"""
    CONTACT_GROUP_ADDITIVE_INHERITANCE = "cg_additive_inheritance"
    """Enables or disables contact group additive inheritance. (bool)"""
    EVENT_HANDLER = "event_handler"
    """Event handler command. (str)"""
    EVENT_HANDLER_ARGUMENTS = "event_handler_arguments"
    """Event handler arguments. (list of str)"""
    EVENT_HANDLER_ENABLED = "event_handler_enabled"
    """Is the event handler enabled? (:ref:`class_general_three_way_option`)"""
    FIRST_NOTIFICATION_DELAY = "first_notification_delay"
    """Notification delay for the first notification, in minutes (int)"""
    FLAP_DETECTION_ENABLED = "flap_detection_enabled"
    """Whether or not flap detection should be enabled. (:ref:`class_general_three_way_option`)"""
    FLAP_DETECTION_OPTIONS = "flap_detection_options"
    # TODO: Check what this option does!!!!
    HOST_NOTIFICATION_OPTIONS = "host_notification_options"
    """Notification option for a host. (:ref:`class_host_notification_option`)"""
    ICON_IMAGE = "icon_image"
    """Icon of the host. (str)"""
    MAX_CHECK_ATTEMPTS = "max_check_attempts"
    """Maximum check attempts. (int)"""
    NAME = "name"
    """Name of the host (str)"""
    NOTES = "notes"
    """Notes regarding the host (str)"""
    NOTIFICATION_ENABLED = "notifications_enabled"
    """Are notification enabled? (:ref:`class_general_three_way_option`)"""
    NOTIFICATION_OPTIONS = "notification_options"
    """Options, when notification should be triggered. (list of :ref:`class_host_notification_option`)"""
    NOTIFICATION_INTERVAL = "notification_interval"
    """Sets the time between notification should be sent, in minutes. (int)"""
    NOTIFICATION_PERIOD = "notification_period"
    """Notification period for the host. (str)"""
    OBSESS_OVER_HOST = "obsess_over_host"
    """Determines whether or not the obsess over host is enabled. (:ref:`class_general_three_way_option`)"""
    PASSIVE_CHECKS_ENABLED = "passive_checks_enabled"
    """Are passive checks enabled? (:ref:`class_general_three_way_option`)"""
    PROCESS_PERF_DATA = "process_perf_data"
    # TODO: Check what this option does!!!
    RETAIN_STATUS_INFORMATION = "retain_status_information"
    """Determines if status information about the host should be retained after a restart. Only useful if you have 
    enabled to global retain_state_information option"""
    RETAIN_NONSTATUS_INFORMATION = "retain_nonstatus_information"
    """Determines if non-status information about the host should be retained after a restart. Only useful if you have 
    enabled to global retain_state_information option"""
    RETRY_CHECK_INTERVAL = "retry_check_interval"
    """Interval on which the checks should be retried, in minutes (int)"""
    SNMP_COMMUNITY = "snmp_community"
    """SNMP community string, used for SNMP versions 1 and 2. (str)"""
    SNMP_VERSION = "snmp_version"
    """SNMP version which should be used. Choose one of "1", "2c", "3". (str)"""
    STALKING_OPTIONS = "stalking_options"
    """Determines which host states stalking should be enabled. (list of :ref:`class_host_stalking_option`)"""
    STATUSMAP_IMAGE = "statusmap_image"
    """Statusmap image, used by statusmap (str)"""
    TIMEZONE = "timezone"
    """Timezone in which the host is located. Example: "Europe/Berlin" (str)"""
    URL = "notes_url"
    """URL (str)"""


class HostNotificationOption(enum.Enum):
    """This class represents a notification option for a host.

    :Note:
        Option NONE can't be used with other options
    """
    DOWN = "d"
    UNREACHABLE = "u"
    RECOVERY = "r"
    FLAPPING = "f"
    DOWNTIME_SCHEDULED = "s"
    NONE = "n"


class HostStalkingOption(enum.Enum):
    """This class represents a stalking option of a host"""
    UP = "u"
    DOWN = "d"
    UNREACHABLE = "u"



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""


class HostStatus:
    """This class represents a HostStatus object

    :param id_unique: Unique id to identify the host
    :type id_unique: int
    :param name: Name of the host
    :type name: str
    :param alias: Alias of the host
    :type alias: str
    :param address: Address of the host
    :type address: str
    :param state: State of the host
    :type state: str
    :param state_type: State type of the host
    :type state_type: str
    :param output: Output of the host
    :type output: str
    :param max_check_attempts: Maximum check attempts of the host
    :type max_check_attempts: int
    :param check_attempt: Current attempt
    :type check_attempt: str
    :param last_check: Last check time
    :type last_check: str
    :param last_state_change: Last time, the state changed
    :type last_state_change: str
    :param last_hard_state_change: Last time, the state changed in hard type
    :type last_hard_state_change: str
    :param acknowledged: Is the acknowledged flag set?
    :type acknowledged: bool
    :param instance_name: Name of the instance
    :type instance_name: str
    :param criticality: A specific criticity
    :type criticality: str
    """
    def __init__(self, *, id_unique=None, name=None, alias=None, address=None, state=None, state_type=None,
                 output=None, max_check_attempts=None, check_attempt=None, last_check=None, last_state_change=None,
                 last_hard_state_change=None, acknowledged=None, instance_name=None, criticality=None):
        self.id_unique = id_unique
        self.name = name
        self.alias = alias
        self.address = address
        self.state = state
        self.state_type = state_type
        self.output = output
        self.max_check_attempts = max_check_attempts
        self.check_attempt = check_attempt
        self.last_check = last_check
        self.last_state_change = last_state_change
        self.last_hard_state_change = last_hard_state_change
        self.acknowledged = acknowledged
        self.instance_name = instance_name
        self.criticality = criticality



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import json
from json import JSONEncoder


class MyEncoder(JSONEncoder):
    def default(self, o):
        if isinstance(o, object):
            return o.__dict__
        else:
            return json.JSONEncoder.default(self, o)


def pack_locals(kwargs):
    """This method is used to pack the locals to another dict

    :param kwargs: locals to process
    :type kwargs: dict

    :return: Returns dict with variables from local_dict
    :rtype: dict
    """
    ret_dict = {}
    for item in kwargs:
        if kwargs[item] and not item == "self":
            if isinstance(kwargs[item], list):
                kwargs[item] = MyEncoder().encode(kwargs[item])
            ret_dict[item] = kwargs[item]
    return ret_dict


def check_if_empty_list(response) -> bool:
    """This method is used to check if the result in a dict is an empty list

    :param response: Response get from the centreon server
    :type response: dict

    :return: Returns True, if response["result"] is an empty list
    :rtype: bool
    """
    if response:
        if "result" in response:
            if isinstance(response["result"], list):
                if len(response["result"]) == 0:
                    return True
    return False


def replace_keys_from_dict(key_dict, dict_to_use):
    for item in key_dict:
        dict_to_use = _replace_keys_from_dict(item, key_dict[item], dict_to_use)
    return dict_to_use


def _replace_keys_from_dict(old_key, new_key, layer):

    if isinstance(layer, list):
        for items in layer:
            _replace_keys_from_dict(old_key, new_key, items)

    elif isinstance(layer, dict):
        to_delete = []

        for key in layer:
            if key == old_key:
                to_delete.append(key)
            _replace_keys_from_dict(old_key, new_key, layer[key])

        for key in to_delete:
            layer[new_key] = layer[old_key]
            del layer[key]

    return layer



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


"""

from centreon_sdk.util import method_utils


class FieldBuilder:
    """This class is used to build field queries. Specify all fields as kwargs, you want to receive.

    Example:
        ```FieldBuilder(id=True, output=True).build()```

        Returns the str: ```id,output```
    """
    def __init__(self, **kwargs):
        self.args = {}
        for item in kwargs:
            self.args[item] = kwargs[item]

    def build(self):
        """This method is used to build the field query string

        :return: Returns the field query string
        :rtype: str
        """
        var_dict = method_utils.replace_keys_from_dict("id_unique", "id", self.args)
        return ",".join([item for item in var_dict if var_dict[item]])



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""


class RealTimeAcknowledgement:
    """This class represents a realtime acknowledgement

    :param id_unique: ID of the acknowledgement
    :type id_unique: int
    :param host_name: Name of the host
    :type host_name: str
    :param entry_time: Beginning of the acknowledgement
    :type entry_time: str
    :param author: Author of the acknowledgement
    :type author: str
    :param comment_data: Short description of the acknowledgement
    :type comment_data: str
    :param sticky: Acknowledgement will be maintained in case of change of NOT OK Status
    :type sticky: bool
    :param notify_contacts: Notification send to the contacts linked to the object
    :type notify_contacts: bool
    :param persistent_comment: Acknowledgement will be maintained in the case of a restart of the scheduler
    :type persistent_comment: bool
    """
    def __init__(self, id_unique, host_name, entry_time, author, comment_data, sticky,
                 notify_contacts, persistent_comment, service_name=None):
        self.id_unique = id_unique
        self.host_name = host_name
        self.entry_time = entry_time
        self.author = author
        self.comment_data = comment_data
        self.sticky = sticky
        self.notify_contacts = notify_contacts
        self.persistent_comment = persistent_comment
        self.service_name = service_name



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class CentBrokerCFG:
    """This class represents a centreon broker configuration

    :param id_unique: ID of the centreon broker configuration
    :type id_unique: int
    :param name: Name of the configuration
    :type name: str
    :param instance: Instance that is linked to the centreon broker configuration
    :type instance: str
    """
    def __init__(self, id_unique, name, instance):
        self.id_unique = id_unique
        self.name = name
        self.instance = instance


class CentBrokerCFGParam(enum.Enum):
    """This class represents the available parameters"""
    FILENAME = "filename"
    """Filename of the configuration (.xml extension) (str)"""
    NAME = "name"
    """Name of the configuration (str)"""
    INSTANCE = "instance"
    """Instance that is linked to the configuration (str)"""
    EVENT_QUEUE_MAX_SIZE = "event_queue_max_size"
    """Event queue maximum size (when number is reached, temporary output will be used) (int)"""
    CACHE_DIRECTORY = "cache_directory"
    """Path for cache files (str)"""
    DAEMON = "daemon"
    """Link the configuration to sbd service (bool)"""
    CORRELATION_ACTIVATE = "correlation_activate"
    """Enable correlation (bool)"""


class CentBrokerCFGInputNature(enum.Enum):
    FILE = "file"
    IPV4 = "ipv4"
    IPV6 = "ipv6"


class CentBrokerCFGOutputNature(enum.Enum):
    IPV4 = "ipv4"
    IPV6 = "ipv6"
    FILE = "file"
    RRD = "rrd"
    STORAGE = "storage"
    SQL = "sql"


class CentBrokerCFGLoggerNature(enum.Enum):
    FILE = "file"
    STANDARD = "standard"
    SYSLOG = "syslog"
    MONITORING = "monitoring"


class CentBrokerCFGIOType(enum.Enum):
    INPUT = "input"
    OUTPUT = "output"
    LOGGER = "logger"



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class ACLResource:
    """This class represents an ACL resource

    :param id_unique: ID of the ACL resource
    :type id_unique: int
    :param name: Name of the ACL resource
    :type name: str
    :param alias: Alias of the ACL resource
    :type alias: str
    :param comment: Comment of the ACL resource
    :type comment: str
    :param activate: Is the ACL resource enabled?
    :type activate: bool
    """
    def __init__(self, id_unique, name, alias, comment, activate):
        self.id_unique = id_unique
        self.name = name
        self.alias = alias
        self.comment = comment
        self.activate = activate


class ACLResourceParam(enum.Enum):
    """This class represents the available parameter for a :ref:`class_acl_resource`"""
    NAME = "name"
    """Name of the ACL resource (str)"""
    ALIAS = "alias"
    """Alias of the ACL resource (str)"""
    ACTIVATE = "activate"
    """Is the ACL resource enabled? (bool)"""


class ACLResourceGrantAction(enum.Enum):
    """This class represents the available grant actions in the format (command: str, Wildcard supported: bool)"""
    GRANT_HOST = ("grant_host", True)
    GRANT_HOSTGROUP = ("grant_hostgroup", True)
    GRANT_SERVICEGROUP = ("grant_servicegroup", True)
    GRANT_METASERVICE = ("grant_metaservice", False)
    ADDHOSTEXCLUSION = ("addhostexclusion", False)
    ADDFILTER_INSTANCE = ("addfilter_instance", False)
    ADDFILTER_HOSTCATEGORY = ("addfilter_hostcategory", False)
    ADDFILTER_SERVICECATEGORY = ("addfilter_servicecategory", False)
    # TODO: Refactor


class ACLResourceRevokeAction(enum.Enum):
    """This class represents the available revoke actions"""
    REVOKE_HOST = "revoke_host"
    REVOKE_HOSTGROUP = "revoke_hostgroup"
    REVOKE_SERVICEGROUP = "revoke_servicegroup"
    REVOKE_METASERVICE = "revoke_metaservice"
    DELHOSTEXCLUSION = "delhostexclusion"
    DELFILTER_INSTANCE = "delfilter_instance"
    DELFILTER_HOSTCATEGORY = "delfilter_hostcategory"
    DELFILTER_SERVICECATEGORY = "delfilter_service_category"
    # TODO: Refactor



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class ServiceTemplate:
    """This class represents a service template

    :param id_unique: ID of the service template
    :type id_unique: int
    :param description: Description of the service template
    :type description: str
    :param alias: Alias of the service template
    :type alias: str
    :param check_command: Check command of the service template
    :type check_command: str
    :param check_command_arg: Arguments of the check commands
    :type check_command_arg: list of str
    :param normal_check_interval: Normal check interval
    :type normal_check_interval: str
    :param retry_check_interval: Retry check interval
    :type retry_check_interval: str
    :param max_check_attempts: Maximum check attempts
    :type max_check_attempts: int
    :param active_checks_enabled: Are active checks enabled?
    :type active_checks_enabled: bool
    :param passive_checks_enabled: Are passive checks enabled?
    :type passive_checks_enabled: bool
    """
    def __init__(self, id_unique, description, alias, check_command, check_command_arg, normal_check_interval,
                 retry_check_interval, max_check_attempts, active_checks_enabled, passive_checks_enabled):
        self.id_unique = id_unique
        self.description = description
        self.alias = alias
        self.check_command = check_command
        self.check_command_arg = check_command_arg
        self.normal_check_interval = normal_check_interval
        self.retry_check_interval = retry_check_interval
        self.max_check_attempts = max_check_attempts
        self.active_checks_enabled = active_checks_enabled
        self.passive_checks_enabled = passive_checks_enabled


class ServiceTemplateParam(enum.Enum):
    """This class represents the available parameters for a service template"""
    ACTIVATE = "activate"
    DESCRIPTION = "description"
    ALIAS = "alias"
    TEMPLATE = "template"
    IS_VOLATILE = "is_volatile"
    CHECK_PERIOD = "check_period"
    CHECK_COMMAND = "check_command"
    CHECK_COMMAND_ARGUMENTS = "check_arguments_arguments"
    MAX_CHECK_ATTEMPTS = "max_check_attempts"
    NORMAL_CHECK_INTERVAL = "normal_check_interval"
    RETRY_CHECK_INTERVAL = "retry_check_interval"
    ACTIVE_CHECKS_ENABLED = "active_checks_enabled"
    PASSIVE_CHECKS_ENABLED = "passive_checks_enabled"
    CONTACT_ADDITIVE_INHERITANCE = "contact_additive_inheritance"
    CONTACT_GROUP_ADDITIVE_INHERITANCE = "cg_additive_inheritance"
    NOTIFICATION_INTERVAL = "notification_interval"
    NOTIFICATION_PERIOD = "notification_period"
    NOTIFICATION_OPTIONS = "notification_options"
    FIRST_NOTIFICATION_DELAY = "first_notification_delay"
    RECOVERY_NOTIFICATION_DELAY = "recovery_notification_delay"
    PARALLELIZE_CHECK = "parallelize_check"
    OBSESS_OVER_SERVICE = "obsess_over_service"
    CHECK_FRESHNESS = "check_freshness"
    FRESHNESS_THRESHOLD = "freshness_threshold"
    EVENT_HANDLER_ENABLED = "event_handler_enabled"
    FLAP_DETECTION_ENABLED = "flap_detection_enabled"
    PROCESS_PERF_DATA = "process_perf_data"
    RETAIN_STATUS_INFORMATION = "retain_status_information"
    RETAIN_NONSTATUS_INFORMATION = "retain_nonstatus_information"
    STALKING_OPTIONS = "stalking_options"
    EVENT_HANDLER = "event_handler"
    EVENT_HANDLER_ARGUMENTS = "event_handler_arguments"
    NOTES = "notes"
    NOTES_URL = "notes_url"
    ACTION_URL = "action_url"
    ICON_IMAGE = "icon_image"
    ICON_IMAGE_ALT = "icon_image_alt"
    GRAPH_TEMPLATE = "graphtemplate"
    COMMENT = "comment"
    SERVICE_NOTIFICATION_OPTIONS = "service_notification_options"


class ServiceTemplateNotificationOption(enum.Enum):
    """This class represents the available notification options"""
    OK = "o"
    WARNING = "w"
    UNKNOWN = "u"
    CRITICAL = "c"
    RECOVERY = "r"
    FLAPPING = "f"
    DOWNTIME_SCHEDULED = "s"


class ServiceTemplateStalkingOption(enum.Enum):
    """This class represents the available stalking options"""
    OK = "o"
    WARNING = "w"
    UNKNOWN = "u"
    CRITICAL = "c"



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""


class HostBuilder:
    """This class is used to simplify the generation of host add string

    :param host_name: Name of the host
    :type host_name: str
    :param host_alias: Optional: Alias of the host
    :type host_alias: str
    :param host_address: Optional: Address of the host
    :type host_address: str
    :param host_template: Optional: Templates of the host
    :type host_template: list of str
    :param instance: Optional: Instance from which the check is performed
    :type instance: str
    :param host_group: Optional: Host groups of the host
    :type host_group: list of str
    """
    def __init__(self, host_name, instance, host_alias="", host_address="", host_template=[], host_group=[]):
        self.host_name = host_name
        self.host_alias = host_alias
        self.host_address = host_address
        self.host_template = host_template
        self.instance = instance
        self.host_group = host_group

    def build(self):
        """This method is used to build the host add string

        :return: Host add string
        :rtype: str
        """
        for item in self.__dict__:
            if isinstance(self.__dict__[item], list):
                self.__dict__[item] = "|".join(self.__dict__[item])
        ret_str = ";".join(self.__dict__.values())
        return ret_str



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class Settings:
    """This class represents the settings

    :param broker: Broker engine
    :type broker: str
    :param broker_correlator_script: Refers to the centreon broker init script
    :type broker_correlator_script: str
    :param centstorage: Enable/Disable CentStorage
    :type centstorage: bool
    :param debug_auth: Enable/Disable authentication debug
    :type debug_auth: bool
    :param debug_ldap_import: Enable/Disable LDAP debug
    :type debug_ldap_import: bool
    :param debug_path: Debug log files directory
    :type debug_path: str
    :param debug_rrdtool: Enable/Disable RRDTool debug
    :type debug_rrdtool: bool
    :param enable_autologin: Enable/Disable autologin
    :type enable_autologin: bool
    :param enable_gmt: Enable/Disable GMT management
    :type enable_gmt: bool
    :param enable_logs_sync: Enable/Disable CentCore log synchronization
    :type enable_logs_sync: bool
    :param enable_perfdata_sync: Enable/Disable perfdata synchronization
    :type enable_perfdata_sync: bool
    :param gmt: GMT timezone of the monitoring system
    :type gmt: str
    :param interval_length: Monitoring interval length in seconds
    :type interval_length: int
    :param mailer_path_bin: Mail client bin path
    :type mailer_path_bin: str
    :param nagios_path_img: Nagios image path
    :type nagios_path_img: str
    :param perl_library_path: Perl library path
    :type perl_library_path: str
    :param rrdtool_path_bin: RRDTool bin path
    :type rrdtool_path_bin: str
    :param snmpttconvertmib_path_bin: SNMPTTT mib converter bin path
    :type snmpttconvertmib_path_bin: str
    :param snmptt_unknowntrap_log_file: SNMPTT unkown trap log file
    :type snmptt_unknowntrap_log_file: str
    """
    def __init__(self, broker, broker_correlator_script, centstorage, debug_auth, debug_ldap_import, debug_path,
                 debug_rrdtool, enable_autologin, enable_gmt, enable_logs_sync, enable_perfdata_sync, gmt,
                 interval_length, mailer_path_bin, nagios_path_img, perl_library_path, rrdtool_path_bin,
                 snmpttconvertmib_path_bin, snmptt_unknowntrap_log_file):
        self.broker = broker
        self.broker_correlation_script = broker_correlator_script
        self.centstorage = centstorage
        self.debug_auth = debug_auth
        self.debug_ldap_import = debug_ldap_import
        self.debug_path = debug_path
        self.debug_rrdtool = debug_rrdtool
        self.enable_autologin = enable_autologin
        self.enable_gmt = enable_gmt
        self.enable_logs_sync = enable_logs_sync
        self.enable_perfdata_sync = enable_perfdata_sync
        self.gmt = gmt
        self.interval_length = interval_length
        self.mailer_path_bin = mailer_path_bin
        self.nagios_path_img = nagios_path_img
        self.perl_library_path = perl_library_path
        self.rrdtool_path_bin = rrdtool_path_bin
        self.snmpttconvertmib_path_bin = snmpttconvertmib_path_bin
        self.snmptt_unknowntrap_log_file = snmptt_unknowntrap_log_file


class SettingsParam(enum.Enum):
    """This class represents a parameter for the settings"""
    BROKER = "broker"
    BROKER_CORRELATOR_SCRIPT = "broker_correlator_script"
    CENTSTORAGE = "centstorage"
    DEBUG_AUTH = "debug_auth"
    DEBUG_LDAP_IMPORT = "debug_ldap_import"
    DEBUG_PATH = "debug_path"
    DEBUG_RRDTOOL = "debug_rrdtool"
    ENABLE_AUTOLOGIN = "enable_autologin"
    ENABLE_GMT = "enable_gmt"
    ENABLE_LOGS_SYNC = "enable_logs_sync"
    ENABLE_PERFDATA_SYNC = "enable_perfdata_sync"
    GMT = "gmt"
    INTERVAL_LENGTH = "interval_length"
    MAILER_PATH_BIN = "mailer_path_bin"
    NAGIOS_PATH_IMG = "nagios_path_img"
    PERL_LIBRARY_PATH = "perl_library_path"
    RRDTOOL_PATH_BIN = "rrdtool_path_bin"
    SNMPTTCONVERTMIB_PATH_BIN = "snmpttconvertmib_path_bin"
    SNMPTT_UNKNOWNTRAP_LOG_FILE = "snmptt_unknowntrap_log_file"



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class Trap:
    """This class represents a trap

    :param id_unique: ID of the trap
    :type id_unique: int
    :param name: Name of the trap
    :type name: str
    :param oid: OID of the trap
    :type oid: str
    :param manufacturer: Manufacturer of the trap
    :type manufacturer: str
    """
    def __init__(self, id_unique, name, oid, manufacturer):
        self.id_unique = id_unique
        self.name = name
        self.oid = oid
        self.manufacturer = manufacturer


class TrapParam(enum.Enum):
    """This class represents the parameter of a trap"""
    NAME = "name"
    COMMENTS = "comments"
    OUTPUT = "output"
    OID = "oid"
    STATUS = "status"
    VENDOR = "vendor"
    MATCHING_MODE = "matching_mode"
    RESCHEDULE_SVC_ENABLE = "reschedule_svc_enable"
    EXECUTION_COMMAND = "execution_command"
    EXECUTION_COMMAND_ENABLE = "execution_command_enable"
    SUBMIT_RESULT_ENABLE = "submit_result_enable"


class TrapMatching:
    """This class represents a matching rule of a trap

    :param id_unique: ID of the matching rule
    :type id_unique: int
    :param string: String of the matching rule
    :type string: str
    :param regexp: Matching regular expression
    :type regexp: str
    :param status: Status to submit
    :type status: str
    :param order: Priority order of the matching rule
    :type order: int
    """
    def __init__(self, id_unique, string, regexp, status, order):
        self.id_unique = id_unique
        self.string = string
        self.regexp = regexp
        self.status = status
        self.order = order


class TrapMatchingParam(enum.Enum):
    """This class represents a parameter of a matching rule of a trap"""
    STRING = "string"
    ORDER = "order"
    STATUS = "status"
    REGEXP = "regexp"



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class ACLAction:
    """This class represents a ACLAction

    :param id_unique: ID of the ACLAction
    :type id_unique: int
    :param name: Name of the ACLAction
    :type name: str
    :param description: Description of the ACLAction
    :type description: str
    :param activate: Is the ACLAction enabled?
    :type activate: bool
    """
    def __init__(self, id_unique, name, description, activate):
        self.id_unique = id_unique
        self.name = name
        self.description = description
        self.activate = activate


class ACLActionParam(enum.Enum):
    """This class represents an acl action parameter"""
    NAME = "name"
    """Name of the acl action rule (str)"""
    DESCRIPTION = "description"
    """Description of the acl action rule (str)"""
    ACTIVATE = "activate"
    """Is the acl action enabled? (bool)"""


class ACLActionRule(enum.Enum):
    """This class represents the available action rules you can grant/revoke

    :Note:
        To grant a rule use True, to revoke False"""
    GLOBAL_EVENT_HANDLER = "global_event_handler"
    """Allow users to enable/disable event handler"""
    GLOBAL_FLAP_DETECTION = "global_flap_detection"
    """Allow users to enable/disable flap detection"""
    GLOBAL_HOST_CHECKS = "global_host_checks"
    """Allow users to enable/disable host checks"""
    GLOBAL_HOST_OBSESS = "global_host_obsess"
    """Allow users to enable/disable obsessive host checks"""
    GLOBAL_HOST_PASSIVE_CHECKS = "global_host_passive_checks"
    """Allow users to enable/disable passive host checks"""
    GLOBAL_NOTIFICATIONS = "global_notifications"
    """Allow users to enable/disable notifications"""
    GLOBAL_PERF_DATA = "global_perf_data"
    """Allow users to enable/disable performance data processing"""
    GLOBAL_RESTART = "global_restart"
    """Allow users to restart the monitoring engine"""
    GLOBAL_SERVICE_CHECKS = "global_service_checks"
    """Allow users to enable/disable service checks"""
    GLOBAL_SERVICE_OBSESS = "global_service_obsess"
    """Allow users to enable/disable obsessive service checks"""
    GLOBAL_SERVICE_PASSIVE_CHECKS = "global_service_passive_checks"
    """Allow users to enable/disable passive service checks"""
    GLOBAL_SHUTDOWN = "global_shutdown"
    """Allow users to shutdown the monitoring engine"""
    HOST_ACKNOWLEDGEMENT = "host_acknowledgement"
    """Allow users to acknowledge a host"""
    HOST_CHECKS = "host_checks"
    """Allow users to enable/disable checks for a host"""
    HOST_CHECKS_FOR_SERVICES = "host_checks_for_services"
    """Allow users to enable/disable all service checks for a host"""
    HOST_COMMENT = "host_comment"
    """Allow users to add/delete a comment for a host"""
    HOST_EVENT_HANDLER = "host_event_handler"
    """Allow users to enable/disable a event handler for a host"""
    HOST_FLAP_DETECTION = "host_flap_detection"
    """Allow users to enable/disable flap detection for a host"""
    HOST_NOTIFICATIONS = "host_notifications"
    """Allow users to enable/disable notifications for a host"""
    HOST_NOTIFICATIONS_FOR_SERVICES = "host_notifications_for_services"
    """Allow users to enable/disable notifications for all services by a host"""
    HOST_SCHEDULE_CHECK = "host_schedule_check"
    """Allow users to schedule a check for a host"""
    HOST_SCHEDULE_DOWNTIME = "host_schedule_downtime"
    """Allow users to schedule a downtime for a host"""
    HOST_SCHEDULE_FORCED_CHECK = "host_schedule_forced_check"
    """Allow users to schedule a forced check for a host"""
    HOST_SUBMIT_RESULT = "host_submit_result"
    """Allow users to submit a result for a host"""
    POLLER_LISTING = "poller_listing"
    """The poller filter will be available to users in the monitoring consoles"""
    POLLER_STATS = "poller_stats"
    """The monitoring poller status overview will be displayed at the top of all pages"""
    SERVICE_ACKNOWLEDGEMENT = "service_acknowledgement"
    """Allow users to acknowledge a service"""
    SERVICE_CHECKS = "service_checks"
    """Allow users to enable/disable service checks"""
    SERVICE_COMMENT = "service_comment"
    """Allow users to add or remove a comment to/from a service"""
    SERVICE_EVENT_HANDLER = "service_event_handler"
    """Allow users to enable/disable event handlers for a service"""
    SERVICE_FLAP_DETECTION = "service_flap_detection"
    """Allow users to enable/disable flap detection for a service"""
    SERVICE_NOTIFICATIONS = "service_notifications"
    """Allow users to enable/disable notifications for a service"""
    SERVICE_PASSIVE_CHECKS = "service_passive_checks"
    """Allow users to enable/disable passive checks for a service"""
    SERVICE_SCHEDULE_CHECK = "service_schedule_check"
    """Allow users to schedule a check for a service"""
    SERVICE_SCHEDULE_DOWNTIME = "service_schedule_downtime"
    """Allow users to schedule a downtime for a service"""
    SERVICE_SCHEDULE_FORCED_CHECK = "service_schedule_forced_check"
    """Allow users to schedule a forced check for a service"""
    SERVICE_SUBMIT_RESULT = "service_submit_result"
    """Allow users to submit results for a service"""
    TOP_COUNTER = "top_counter"
    """The monitoring overview will be displayed at the top of all pages"""



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class ResourceCFG:
    """This class represents a resource variable

    :param name: Name of the variable
    :type name: str
    :param value: Value of the variable
    :type value: str
    :param comment: Comment of the variable
    :type comment: str
    :param activate: Is the variable enabled?
    :type activate: bool
    :param instance: List of the instances the variable is linked to
    :type instance: list of str
    :param id_unique: ID of the variable
    :type id_unique: int
    """
    def __init__(self, name, value, comment, activate, instance, id_unique):
        self.id_unique = id_unique
        self.name = name
        self.value = value
        self.comment = comment
        self.activate = activate
        self.instance = instance


class ResourceCFGParam(enum.Enum):
    """This class represents the parameters, available for resource cfgs"""
    NAME = "name"
    VALUE = "value"
    ACTIVATE = "activate"
    COMMENT = "comment"
    INSTANCE = "instance"



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""


class RealTimeDowntimeHost:
    def __init__(self, host_name, author, actual_start_time, actual_end_time, start_time, end_time, comment_data,
                 duration, fixed, url, id_unique):
        """This class represents a realtime downtime for a host

        :param host_name: Name of the host
        :type host_name: str
        :param author: Author of the downtime
        :type author: str
        :param actual_start_time: Actual start time in case of a flexible downtime
        :type actual_start_time: str
        :param actual_end_time: Actual end time in case of a flexible downtime
        :type actual_end_time: str
        :param start_time: Beginning of the downtime
        :type start_time: str
        :param end_time: End of the downtime
        :type end_time: str
        :param comment_data: Short description of the realtime downtime
        :type comment_data: str
        :param duration: Duration of the downtime
        :type duration: str
        :param fixed: Is the downtime fixed or flexible
        :type fixed: bool
        :param url: URL to the host
        :type url: str
        :param id_unique: ID of the downtime
        :type id_unique: int
        """
        self.host_name = host_name
        self.author = author
        self.actual_start_time = actual_start_time
        self.actual_end_time = actual_end_time
        self.start_time = start_time
        self.end_time = end_time
        self.comment_data = comment_data
        self.duration = duration
        self.fixed = fixed
        self.url = url
        self.id_unique = id_unique


class RealTimeDowntimeService:
    def __init__(self, host_name, service_name, author, actual_start_time, actual_end_time, start_time, end_time,
                 comment_data, duration, fixed, url, id_unique):
        """This class represents a realtime downtime for a host

        :param host_name: Name of the host
        :type host_name: str
        :param service_name: Service of the host
        :type service_name: str
        :param author: Author of the downtime
        :type author: str
        :param actual_start_time: Actual start time in case of a flexible downtime
        :type actual_start_time: str
        :param actual_end_time: Actual end time in case of a flexible downtime
        :type actual_end_time: str
        :param start_time: Beginning of the downtime
        :type start_time: str
        :param end_time: End of the downtime
        :type end_time: str
        :param comment_data: Short description of the realtime downtime
        :type comment_data: str
        :param duration: Duration of the downtime
        :type duration: str
        :param fixed: Is the downtime fixed or flexible
        :type fixed: bool
        :param url: URL to the host
        :type url: str
        :param id_unique: ID of the downtime
        :type id_unique: int
        """
        self.host_name = host_name
        self.service_name = service_name
        self.author = author
        self.actual_start_time = actual_start_time
        self.actual_end_time = actual_end_time
        self.start_time = start_time
        self.end_time = end_time
        self.comment_data = comment_data
        self.duration = duration
        self.fixed = fixed
        self.url = url
        self.id_unique = id_unique



import setuptools

with open("README.md") as f:
    long_description = f.read()

setuptools.setup(
    name="centreon-sdk",
    version="0.1.5",
    author="Niklas Pfister",
    author_email="contact@omikron.pw",
    description="A SDK for python to communicate with Centreon",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/myOmikron/PythonCentreonSDK",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "wheel",
        "requests"
    ]
)



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class Downtime:
    """This class represents a recurrent downtime

    :param id_unique: ID of the downtime
    :type id_unique: int
    :param name: Name of the downtime
    :type name: str
    :param description: Description of the downtime
    :type description: str
    :param activate: Is the downtime activated?
    :type activate: bool
    :param hosts: Optional: Hosts the downtime should apply to
    :type hosts: list of str
    :param services: Optional: Services the downtime should apply to
    :type services: list of str
    :param service_groups: Optional: Service groups the downtime should apply to
    :type service_groups: list of str
    :param host_groups: Optional: Host groups the downtime should apply to
    :type host_groups: list of str
    """
    def __init__(self, id_unique, name, description, activate, hosts=None, services=None, service_groups=None,
                 host_groups=None):
        self.id_unique = id_unique
        self.name = name
        self.description = description
        self.activate = activate
        self.hosts = hosts
        self.host_groups = host_groups
        self.services = services
        self.service_groups = service_groups


class DowntimeType(enum.Enum):
    """This class represents the available types of recurrent downtimes"""
    HOST = "host"
    HOST_GROUP = "hg"
    SERVICE = "service"
    SERVICE_GROUP = "sg"


class DowntimeParam(enum.Enum):
    """This class represents the available parameter for a recurrent downtime"""
    NAME = "name"
    DESCRIPTION = "description"


class DowntimePeriod:
    """This class represents a period of a recurrent downtime

    :param position: Position of the period
    :type position: int
    :param start_time: Start time of the downtime. Format hh:mm:ss
    :type start_time: str
    :param end_time: End time of the downtime. Format hh:mm:ss
    :type end_time: str
    :param fixed: Is the downtime fixed or flexible? Fixed = True, Flexible = False
    :type fixed: bool
    :param duration: Duration of the downtime in seconds
    :type duration: int
    :param day_of_week: Days of the week the period should apply
    :type day_of_week: list of int
    :param day_of_month: Days of the month the period should apply
    :type day_of_month: list of int
    :param month_cycle: First or last
    :type month_cycle: str
    """
    def __init__(self, position, start_time, end_time, fixed, duration, day_of_week, day_of_month, month_cycle):
        self.position = position
        self.start_time = start_time
        self.end_time = end_time
        self.fixed = fixed
        self.duration = duration
        self.day_of_week = day_of_week
        self.day_of_month = day_of_month
        self.month_cycle = month_cycle



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import urllib3

from centreon_sdk.builder.field_builder import FieldBuilder
from centreon_sdk.builder.host_builder import HostBuilder
from centreon_sdk.centreon import Centreon
from centreon_sdk.network.network import HTTPVerb
from centreon_sdk.objects.base.acl_group import ACLGroupParam
from centreon_sdk.objects.base.host import HostParam

if __name__ == '__main__':
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
    centreon = Centreon("REST", "1lCNqIvU", "https://centreon.omikron.pw/centreon/api/index.php", verify=False)
    result = centreon.real_time_acknowledgement_show_host()
    if isinstance(result, list):
        for item in result:
            print(item.__dict__)
    else:
        print(result)



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class Instance:
    """This class represents an instance also known as a poller

    :param id_unique: ID of the instance
    :type id_unique: int
    :param name: Name of the instance
    :type name: str
    :param localhost: Is the poller the main poller?
    :type localhost: bool
    :param ip_address: IP address of the poller
    :type ip_address: str
    :param activate: Is the poller enabled?
    :type activate: bool
    :param status: Is the poller running?
    :type status: bool
    :param engine_restart_command: Engine restart command
    :type engine_restart_command: str
    :param engine_reload_command: Engine reload command
    :type engine_reload_command: str
    :param broker_reload_command: Broker reload command
    :type broker_reload_command: str
    :param bin_scheduler: Path of the scheduler binary
    :type bin_scheduler: str
    :param stats_bin: Path of the nagios stats binary
    :type stats_bin: str
    :param ssh_port: Port of the ssh server
    :type ssh_port: int
    """
    def __init__(self, id_unique, name, localhost, ip_address, activate, status, engine_restart_command,
                 engine_reload_command, broker_reload_command, bin_scheduler, stats_bin,
                 ssh_port):
        self.id_unique = id_unique
        self.name = name
        self.localhost = localhost
        self.ip_address = ip_address
        self.activate = activate
        self.status = status
        self.engine_restart_command = engine_restart_command
        self.engine_reload_command = engine_reload_command
        self.broker_reload_command = broker_reload_command
        self.bin_scheduler = bin_scheduler
        self.stats_bin = stats_bin
        self.ssh_port = ssh_port


class InstanceParam(enum.Enum):
    """This class represents the parameters available for an instance"""
    NAME = "name"
    LOCALHOST = "localhost"
    IP_ADDRESS = "ns_ip_address"
    ACTIVATE = "ns_activate"
    ENGINE_START_COMMAND = "engine_start_command"
    ENGINE_STOP_COMMAND = "engine_stop_command"
    ENGINE_RESTART_COMMAND = "engine_restart_command"
    ENGINE_RELOAD_COMMAND = "engine_reload_command"
    NAGIOS_BIN = "nagios_bin"
    NAGIOS_STATS_BIN = "nagiosstats_bin"
    SSH_PORT = "ssh_port"
    BROKER_RELOAD_COMMAND = "broker_reload_command"
    CENTREON_BROKER_CFG_PATH = "centreonbroker_cfg_path"
    CENTREON_BROKER_MODULE_PATH = "centreonbroker_module_path"



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum


class Dependency:
    """This class represents a dependency

    :param id_unique: ID of the dependency
    :type id_unique: int
    :param name: Name of the dependency
    :type name: str
    :param description: Description of the dependency
    :type description: str
    :param inherits_parent: Inherits the dependency from its parent?
    :type inherits_parent: bool
    :param execution_failure_criteria: Defines which parent states prevent dependent resources from being checked
    :type execution_failure_criteria: :ref:`class_failure_criteria`
    :param notification_failure_criteria: Defines which parent states prevent notifications on dependent resources
    :type notification_failure_criteria: :ref:`class_failure_criteria`
    """
    def __init__(self, id_unique, name, description, inherits_parent, execution_failure_criteria,
                 notification_failure_criteria):
        self.id_unique = id_unique
        self.name = name
        self.description = description
        self.inherits_parent = inherits_parent
        self.execution_failure_criteria = execution_failure_criteria
        self.notification_failure_criteria = notification_failure_criteria


class FailureCriteria(enum.Enum):
    """This class represents the failure criteria"""
    OK = "o"
    WARNING = "w"
    UNKNOWN = "u"
    CRITICAL = "c"
    PENDING = "p"
    DOWN = "d"
    NONE = "n"


class DependencyType(enum.Enum):
    """This class represents the types available for dependencies"""
    HOST = "HOST"
    HOST_GROUP = "HG"
    SERVICE_GROUP = "SG"
    SERVICE = "SERVICE"
    META_SERVICE = "META"


class DependencyParam(enum.Enum):
    """This class represents the parameter available for dependencies"""
    NAME = "name"
    DESCRIPTION = "description"
    COMMENT = "comment"
    INHERITS_PARENT = "inherits_parent"
    EXECUTION_FAILURE_CRITERIA = "execution_failure_criteria"
    NOTIFICATION_FAILURE_CRITERIA = "notification_failure_criteria"



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""


class ServiceStatus:
    """This class represents the status of a service

    :param name: Name of the host
    :type name: str
    :param alias: Alias of the host
    :type alias: str
    :param address: Address of the host
    :type address: str
    :param state: State of the host
    :type state: str
    :param state_type: State type of the host
    :type state_type: str
    :param output: Output of the service
    :type output: str
    :param max_check_attempts: Max check attempts
    :type max_check_attempts: int
    :param check_attempt: Check attempt
    :type check_attempt: int
    :param last_check: Last check results
    :type last_check: int
    :param last_check_state: Last check state
    :type last_check_state: int
    :param last_state_change: Last state change
    :type last_state_change: int
    :param last_hard_state_change: Last change of type hard
    :type last_hard_state_change: int
    :param acknowledged: Is the acknowledged flag set?
    :type acknowledged: bool
    :param instance_name: Name of the instance
    :type instance_name: str
    :param criticality: What type of criticality is set?
    :type criticality: str
    :param id_unique: Unique id of host
    :type id_unique: int
    """
    def __init__(self, *, name=None, alias=None, address=None, state=None, state_type=None, output=None,
                 max_check_attempts=None, check_attempt=None, last_check=None, last_check_state=None,
                 last_state_change=None, last_hard_state_change=None, acknowledged=None, instance_name=None,
                 criticality=None, id_unique=None):

        self.name = name
        self.alias = alias
        self.address = address
        self.state = state
        self.state_type = state_type
        self.output = output
        self.max_check_attempts = max_check_attempts
        self.check_attempt = check_attempt
        self.last_check = last_check
        self.last_check_state = last_check_state
        self.last_hard_state_change = last_hard_state_change
        self.acknowledged = acknowledged
        self.instance_name = instance_name
        self.criticality = criticality
        self.id_unique = id_unique
        self.last_check_state = last_state_change



"""
This program is a library to communicate with the Centreon REST API

Copyright (C) 2019 Niklas Pfister, contact@omikron.pw

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import enum
import json
import requests

from centreon_sdk.util import method_utils


class HTTPVerb(enum.Enum):
    GET = 1
    POST = 2


class Network:
    """This class is used to manage the network

    :param config: Config to use
    :type config: :ref:object_config:
    """
    def __init__(self, config, verify=True):
        self.config = config
        self.session = requests.Session()
        self.session.verify = verify
        self.replace_keys_dict = {"hg name": "host_group_name",
                                  "hg id": "host_group_id",
                                  "month cycle": "month_cycle",
                                  "day of month": "day_of_month",
                                  "day of week": "day_of_week",
                                  "end time": "end_time",
                                  "start time": "start_time",
                                  "service groups": "service_groups",
                                  "host groups": "host_groups",
                                  "ssh port": "ssh_port",
                                  "stats bin": "stats_bin",
                                  "broker reload command": "broker_reload_command",
                                  "engine reload command": "engine_reload_command",
                                  "engine restart command": "engine_restart_command",
                                  "ip address": "ip_address",
                                  "bin": "bin_scheduler",
                                  "passive checks enabled": "passive_checks_enabled",
                                  "active checks enabled": "active_checks_enabled",
                                  "max check attempts": "max_check_attempts",
                                  "retry check interval": "retry_check_interval",
                                  "normal check interval": "normal_check_interval",
                                  "check command arg": "check_command_arg",
                                  "check command": "check_command",
                                  "host name": "host_name",
                                  "host id": "host_id",
                                  "gui access": "gui_access",
                                  "macro value": "macro_value",
                                  "macro name": "macro_name",
                                  "id": "id_unique",
                                  "type": "cmd_type"}

    def make_request(self, verb, *, params=None, data=None, use_encode_json=True, use_header=True):
        """This method is used to make request to the REST endpoint

        :param verb: HTTP Verb to use
        :type :ref:object_http_verb:
        :param params: Optional: dict to get encoded in url
        :type params: dict
        :param data: Optional: dict to get encoded in body
        :type data: dict
        :param use_encode_json: Optional: Set False to do not use json serialization in data
        :type use_encode_json: bool
        :param use_header: Optional: Set false to do not use header
        :type use_header: bool

        :return: json encoded string
        :rtype: str
        """
        response = None
        header = self.config.vars["header"] if use_header else None
        data = json.dumps(data) if use_encode_json else data

        if verb == HTTPVerb.GET:
            response = self.session.get(self.config.vars["URL"], params=params, headers=header)
        elif verb == HTTPVerb.POST:
            response = self.session.post(self.config.vars["URL"], params=params, data=data, headers=header)

        if not response.status_code == 200:
            print(response.status_code, response.text)
            return
        json_decoded = json.loads(response.text)
        json_decoded = method_utils.replace_keys_from_dict(self.replace_keys_dict, json_decoded)
        return json_decoded

